package java.awt.image;

import java.util.Arrays;

public class ComponentSampleModel extends SampleModel
{
    protected int[] bandOffsets;
    protected int[] bankIndices;
    protected int numBands;
    protected int numBanks;
    protected int scanlineStride;
    protected int pixelStride;
    
    private static native void initIDs();
    
    public ComponentSampleModel(int dataType, int n, int n2, int pixelStride, int scanlineStride, int[] array) {
        super(dataType, n, n2, array.length);
        this.numBands = 1;
        this.numBanks = 1;
        this.dataType = dataType;
        this.pixelStride = pixelStride;
        this.scanlineStride = scanlineStride;
        this.bandOffsets = (int[])(int[])array.clone();
        this.numBands = this.bandOffsets.length;
        if (pixelStride < 0) {
            throw new IllegalArgumentException("Pixel stride must be >= 0");
        }
        if (scanlineStride < 0) {
            throw new IllegalArgumentException("Scanline stride must be >= 0");
        }
        if (this.numBands < 1) {
            throw new IllegalArgumentException("Must have at least one band.");
        }
        if (dataType < 0 || dataType > 5) {
            throw new IllegalArgumentException("Unsupported dataType.");
        }
        this.bankIndices = new int[this.numBands];
        for (int i = 0; i < this.numBands; ++i) {
            this.bankIndices[i] = 0;
        }
        this.verify();
        return;
    }
    
    public ComponentSampleModel(int dataType, int n, int n2, int pixelStride, int scanlineStride, int[] array, int[] array2) {
        super(dataType, n, n2, array2.length);
        this.numBands = 1;
        this.numBanks = 1;
        this.dataType = dataType;
        this.pixelStride = pixelStride;
        this.scanlineStride = scanlineStride;
        this.bandOffsets = (int[])(int[])array2.clone();
        this.bankIndices = (int[])(int[])array.clone();
        if (pixelStride < 0) {
            throw new IllegalArgumentException("Pixel stride must be >= 0");
        }
        if (scanlineStride < 0) {
            throw new IllegalArgumentException("Scanline stride must be >= 0");
        }
        if (dataType < 0 || dataType > 5) {
            throw new IllegalArgumentException("Unsupported dataType.");
        }
        int n3 = this.bankIndices[0];
        if (n3 < 0) {
            throw new IllegalArgumentException("Index of bank 0 is less than 0 (" + n3 + ")");
        }
        for (int i = 1; i < this.bankIndices.length; ++i) {
            if (this.bankIndices[i] > n3) {
                n3 = this.bankIndices[i];
            }
            else if (this.bankIndices[i] < 0) {
                throw new IllegalArgumentException("Index of bank " + i + " is less than 0 (" + n3 + ")");
            }
        }
        this.numBanks = n3 + 1;
        this.numBands = this.bandOffsets.length;
        if (this.bandOffsets.length != this.bankIndices.length) {
            throw new IllegalArgumentException("Length of bandOffsets must equal length of bankIndices.");
        }
        this.verify();
        return;
    }
    
    private void verify() {
        this.getBufferSize();
    }
    
    private int getBufferSize() {
        int max = this.bandOffsets[0];
        for (int i = 1; i < this.bandOffsets.length; ++i) {
            max = Math.max(max, this.bandOffsets[i]);
        }
        if (max < 0 || max > 2147483646) {
            throw new IllegalArgumentException("Invalid band offset");
        }
        if (this.pixelStride < 0 || this.pixelStride > Integer.MAX_VALUE / this.width) {
            throw new IllegalArgumentException("Invalid pixel stride");
        }
        if (this.scanlineStride < 0 || this.scanlineStride > Integer.MAX_VALUE / this.height) {
            throw new IllegalArgumentException("Invalid scanline stride");
        }
        int n = max + 1;
        int n2 = this.pixelStride * (this.width - 1);
        if (n2 > Integer.MAX_VALUE - n) {
            throw new IllegalArgumentException("Invalid pixel stride");
        }
        int n3 = n + n2;
        int n4 = this.scanlineStride * (this.height - 1);
        if (n4 > Integer.MAX_VALUE - n3) {
            throw new IllegalArgumentException("Invalid scan stride");
        }
        return n3 + n4;
    }
    
    int[] orderBands(int[] array, int n) {
        int[] array2 = new int[array.length];
        int[] array3 = new int[array.length];
        for (int i = 0; i < array2.length; ++i) {
            array2[i] = i;
        }
        for (int j = 0; j < array3.length; array3[array2[j]] = j * n, , ++j) {
            for (int k = j + 1; k < array3.length; ++k) {
                if (array[array2[j]] > array[array2[k]]) {
                    j = k;
                }
            }
        }
        return array3;
    }
    
    @Override
    public SampleModel createCompatibleSampleModel(int n, int n2) {
        int min = this.bandOffsets[0];
        int max = this.bandOffsets[0];
        for (int i = 1; i < this.bandOffsets.length; ++i) {
            min = Math.min(min, this.bandOffsets[i]);
            max = Math.max(max, this.bandOffsets[i]);
        }
        int a = max - min;
        int length = this.bandOffsets.length;
        int abs = Math.abs(this.pixelStride);
        int abs2 = Math.abs(this.scanlineStride);
        int abs3 = Math.abs(a);
        int[] array;
        if (abs > abs2) {
            if (abs > abs3) {
                if (abs2 > abs3) {
                    array = new int[this.bandOffsets.length];
                    for (int k = 0; k < length; ++k) {
                        array[k] = this.bandOffsets[k] - min;
                    }
                    abs2 = abs3 + 1;
                    abs = abs2 * n2;
                }
                else {
                    array = this.orderBands(this.bandOffsets, abs2 * n2);
                    abs = length * abs2 * n2;
                }
            }
            else {
                abs = abs2 * n2;
                array = this.orderBands(this.bandOffsets, abs * n);
            }
        }
        else if (abs > abs3) {
            array = new int[this.bandOffsets.length];
            for (int j = 0; j < length; ++j) {
                array[j] = this.bandOffsets[j] - min;
            }
            abs = abs3 + 1;
            abs2 = abs * n;
        }
        else if (abs2 > abs3) {
            array = this.orderBands(this.bandOffsets, abs * n);
            abs2 = length * abs * n;
        }
        else {
            abs2 = abs * n;
            array = this.orderBands(this.bandOffsets, abs2 * n2);
        }
        int n3 = 0;
        if (this.scanlineStride < 0) {
            n3 += abs2 * n2;
            abs2 *= -1;
        }
        if (this.pixelStride < 0) {
            n3 += abs * n;
            abs *= -1;
        }
        for (int l = 0; l < length; ++l) {
            int n4 = l;
            array[n4] += n3;
        }
        return new ComponentSampleModel(this.dataType, n, n2, abs, abs2, this.bankIndices, array);
    }
    
    @Override
    public SampleModel createSubsetSampleModel(int[] array) {
        if (array.length > this.bankIndices.length) {
            throw new RasterFormatException("There are only " + this.bankIndices.length + " bands");
        }
        int[] array2 = new int[array.length];
        int[] array3 = new int[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[i] = this.bankIndices[array[i]];
            array3[i] = this.bandOffsets[array[i]];
        }
        return new ComponentSampleModel(this.dataType, this.width, this.height, this.pixelStride, this.scanlineStride, array2, array3);
    }
    
    @Override
    public DataBuffer createDataBuffer() {
        DataBuffer dataBuffer = null;
        int bufferSize = this.getBufferSize();
        switch (this.dataType) {
            case 0: {
                dataBuffer = new DataBufferByte(bufferSize, this.numBanks);
                break;
            }
            case 1: {
                dataBuffer = new DataBufferUShort(bufferSize, this.numBanks);
                break;
            }
            case 2: {
                dataBuffer = new DataBufferShort(bufferSize, this.numBanks);
                break;
            }
            case 3: {
                dataBuffer = new DataBufferInt(bufferSize, this.numBanks);
                break;
            }
            case 4: {
                dataBuffer = new DataBufferFloat(bufferSize, this.numBanks);
                break;
            }
            case 5: {
                dataBuffer = new DataBufferDouble(bufferSize, this.numBanks);
                break;
            }
        }
        return dataBuffer;
    }
    
    public int getOffset(int n, int n2) {
        return n2 * this.scanlineStride + n * this.pixelStride + this.bandOffsets[0];
    }
    
    public int getOffset(int n, int n2, int n3) {
        return n2 * this.scanlineStride + n * this.pixelStride + this.bandOffsets[n3];
    }
    
    @Override
    public final int[] getSampleSize() {
        int[] array = new int[this.numBands];
        int sampleSize = this.getSampleSize(0);
        for (int i = 0; i < this.numBands; ++i) {
            array[i] = sampleSize;
        }
        return array;
    }
    
    @Override
    public final int getSampleSize(int n) {
        return DataBuffer.getDataTypeSize(this.dataType);
    }
    
    public final int[] getBankIndices() {
        return (int[])(int[])this.bankIndices.clone();
    }
    
    public final int[] getBandOffsets() {
        return (int[])(int[])this.bandOffsets.clone();
    }
    
    public final int getScanlineStride() {
        return this.scanlineStride;
    }
    
    public final int getPixelStride() {
        return this.pixelStride;
    }
    
    @Override
    public final int getNumDataElements() {
        return this.getNumBands();
    }
    
    @Override
    public Object getDataElements(int n, int n2, Object o, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int transferType = this.getTransferType();
        int numDataElements = this.getNumDataElements();
        int n3 = n2 * this.scanlineStride + n * this.pixelStride;
        switch (transferType) {
            case 0: {
                byte[] array;
                if (o == null) {
                    array = new byte[numDataElements];
                }
                else {
                    array = (byte[])(byte[])o;
                }
                for (int i = 0; i < numDataElements; ++i) {
                    array[i] = (byte)dataBuffer.getElem(this.bankIndices[i], n3 + this.bandOffsets[i]);
                }
                o = array;
                break;
            }
            case 1:
            case 2: {
                short[] array2;
                if (o == null) {
                    array2 = new short[numDataElements];
                }
                else {
                    array2 = (short[])(short[])o;
                }
                for (int j = 0; j < numDataElements; ++j) {
                    array2[j] = (short)dataBuffer.getElem(this.bankIndices[j], n3 + this.bandOffsets[j]);
                }
                o = array2;
                break;
            }
            case 3: {
                int[] array3;
                if (o == null) {
                    array3 = new int[numDataElements];
                }
                else {
                    array3 = (int[])(int[])o;
                }
                for (int k = 0; k < numDataElements; ++k) {
                    array3[k] = dataBuffer.getElem(this.bankIndices[k], n3 + this.bandOffsets[k]);
                }
                o = array3;
                break;
            }
            case 4: {
                float[] array4;
                if (o == null) {
                    array4 = new float[numDataElements];
                }
                else {
                    array4 = (float[])(float[])o;
                }
                for (int l = 0; l < numDataElements; ++l) {
                    array4[l] = dataBuffer.getElemFloat(this.bankIndices[l], n3 + this.bandOffsets[l]);
                }
                o = array4;
                break;
            }
            case 5: {
                double[] array5;
                if (o == null) {
                    array5 = new double[numDataElements];
                }
                else {
                    array5 = (double[])(double[])o;
                }
                for (int n4 = 0; n4 < numDataElements; ++n4) {
                    array5[n4] = dataBuffer.getElemDouble(this.bankIndices[n4], n3 + this.bandOffsets[n4]);
                }
                o = array5;
                break;
            }
        }
        return o;
    }
    
    @Override
    public int[] getPixel(int n, int n2, int[] array, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array != null) {
        }
        else {
            array = new int[this.numBands];
        }
        int n3 = n2 * this.scanlineStride + n * this.pixelStride;
        for (int i = 0; i < this.numBands; ++i) {
            array[i] = dataBuffer.getElem(this.bankIndices[i], n3 + this.bandOffsets[i]);
        }
        return array;
    }
    
    @Override
    public int[] getPixels(int n, int n2, int n3, int n4, int[] array, DataBuffer dataBuffer) {
        int n5 = n + n3;
        int n6 = n2 + n4;
        if (n < 0 || n >= this.width || n3 > this.width || n5 < 0 || n5 > this.width || n2 < 0 || n2 >= this.height || n2 > this.height || n6 < 0 || n6 > this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array != null) {
        }
        else {
            array = new int[n3 * n4 * this.numBands];
        }
        int n7 = n2 * this.scanlineStride + n * this.pixelStride;
        int n8 = 0;
        for (int i = 0; i < n4; n7 += this.scanlineStride, ++i) {
            for (int j = 0; j < n3; n7 += this.pixelStride, ++j) {
                for (int k = 0; k < this.numBands; ++k) {
                    array[n8++] = dataBuffer.getElem(this.bankIndices[k], n7 + this.bandOffsets[k]);
                }
            }
        }
        return array;
    }
    
    @Override
    public int getSample(int n, int n2, int n3, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        return dataBuffer.getElem(this.bankIndices[n3], n2 * this.scanlineStride + n * this.pixelStride + this.bandOffsets[n3]);
    }
    
    @Override
    public float getSampleFloat(int n, int n2, int n3, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        return dataBuffer.getElemFloat(this.bankIndices[n3], n2 * this.scanlineStride + n * this.pixelStride + this.bandOffsets[n3]);
    }
    
    @Override
    public double getSampleDouble(int n, int n2, int n3, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        return dataBuffer.getElemDouble(this.bankIndices[n3], n2 * this.scanlineStride + n * this.pixelStride + this.bandOffsets[n3]);
    }
    
    @Override
    public int[] getSamples(int n, int n2, int n3, int n4, int n5, int[] array, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n + n3 > this.width || n2 + n4 > this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array != null) {
        }
        else {
            array = new int[n3 * n4];
        }
        int n6 = n2 * this.scanlineStride + n * this.pixelStride + this.bandOffsets[n5];
        int n7 = 0;
        for (int i = 0; i < n4; n6 += this.scanlineStride, ++i) {
            for (int j = 0; j < n3; ++j) {
                array[n7++] = dataBuffer.getElem(this.bankIndices[n5], n6);
                n6 += this.pixelStride;
            }
        }
        return array;
    }
    
    @Override
    public void setDataElements(int n, int n2, Object o, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int transferType = this.getTransferType();
        int numDataElements = this.getNumDataElements();
        int n3 = n2 * this.scanlineStride + n * this.pixelStride;
        switch (transferType) {
            case 0: {
                byte[] array = (byte[])(byte[])o;
                for (int i = 0; i < numDataElements; ++i) {
                    dataBuffer.setElem(this.bankIndices[i], n3 + this.bandOffsets[i], array[i] & 0xFF);
                }
                break;
            }
            case 1:
            case 2: {
                short[] array2 = (short[])(short[])o;
                for (int j = 0; j < numDataElements; ++j) {
                    dataBuffer.setElem(this.bankIndices[j], n3 + this.bandOffsets[j], array2[j] & 0xFFFF);
                }
                break;
            }
            case 3: {
                int[] array3 = (int[])(int[])o;
                for (int k = 0; k < numDataElements; ++k) {
                    dataBuffer.setElem(this.bankIndices[k], n3 + this.bandOffsets[k], array3[k]);
                }
                break;
            }
            case 4: {
                float[] array4 = (float[])(float[])o;
                for (int l = 0; l < numDataElements; ++l) {
                    dataBuffer.setElemFloat(this.bankIndices[l], n3 + this.bandOffsets[l], array4[l]);
                }
                break;
            }
            case 5: {
                double[] array5 = (double[])(double[])o;
                for (int n4 = 0; n4 < numDataElements; ++n4) {
                    dataBuffer.setElemDouble(this.bankIndices[n4], n3 + this.bandOffsets[n4], array5[n4]);
                }
                break;
            }
        }
        return;
    }
    
    @Override
    public void setPixel(int n, int n2, int[] array, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n3 = n2 * this.scanlineStride + n * this.pixelStride;
        for (int i = 0; i < this.numBands; ++i) {
            dataBuffer.setElem(this.bankIndices[i], n3 + this.bandOffsets[i], array[i]);
        }
        return;
    }
    
    @Override
    public void setPixels(int n, int n2, int n3, int n4, int[] array, DataBuffer dataBuffer) {
        int n5 = n + n3;
        int n6 = n2 + n4;
        if (n < 0 || n >= this.width || n3 > this.width || n5 < 0 || n5 > this.width || n2 < 0 || n2 >= this.height || n4 > this.height || n6 < 0 || n6 > this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n7 = n2 * this.scanlineStride + n * this.pixelStride;
        int n8 = 0;
        for (int i = 0; i < n4; n7 += this.scanlineStride, ++i) {
            for (int j = 0; j < n3; n7 += this.pixelStride, ++j) {
                for (int k = 0; k < this.numBands; ++k) {
                    dataBuffer.setElem(this.bankIndices[k], n7 + this.bandOffsets[k], array[n8++]);
                }
            }
        }
        return;
    }
    
    @Override
    public void setSample(int n, int n2, int n3, int n4, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        dataBuffer.setElem(this.bankIndices[n3], n2 * this.scanlineStride + n * this.pixelStride + this.bandOffsets[n3], n4);
        return;
    }
    
    @Override
    public void setSample(int n, int n2, int n3, float n4, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        dataBuffer.setElemFloat(this.bankIndices[n3], n2 * this.scanlineStride + n * this.pixelStride + this.bandOffsets[n3], n4);
        return;
    }
    
    @Override
    public void setSample(int n, int n2, int n3, double n4, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        dataBuffer.setElemDouble(this.bankIndices[n3], n2 * this.scanlineStride + n * this.pixelStride + this.bandOffsets[n3], n4);
        return;
    }
    
    @Override
    public void setSamples(int n, int n2, int n3, int n4, int n5, int[] array, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n + n3 > this.width || n2 + n4 > this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n6 = n2 * this.scanlineStride + n * this.pixelStride + this.bandOffsets[n5];
        int n7 = 0;
        for (int i = 0; i < n4; n6 += this.scanlineStride, ++i) {
            for (int j = 0; j < n3; n6 += this.pixelStride, ++j) {
                dataBuffer.setElem(this.bankIndices[n5], n6, array[n7++]);
            }
        }
        return;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == null || !(o instanceof ComponentSampleModel)) {
            return false;
        }
        ComponentSampleModel componentSampleModel = (ComponentSampleModel)o;
        return this.width == componentSampleModel.width && this.height == componentSampleModel.height && this.numBands == componentSampleModel.numBands && this.dataType == componentSampleModel.dataType && Arrays.equals(this.bandOffsets, componentSampleModel.bandOffsets) && Arrays.equals(this.bankIndices, componentSampleModel.bankIndices) && this.numBands == componentSampleModel.numBands && this.numBanks == componentSampleModel.numBanks && this.scanlineStride == componentSampleModel.scanlineStride && this.pixelStride == componentSampleModel.pixelStride;
    }
    
    @Override
    public int hashCode() {
        int n = (((this.width << 8 ^ this.height) << 8 ^ this.numBands) << 8 ^ this.dataType) << 8;
        for (int i = 0; i < this.bandOffsets.length; ++i) {
            n = (n ^ this.bandOffsets[i]) << 8;
        }
        for (int j = 0; j < this.bankIndices.length; ++j) {
            n = (n ^ this.bankIndices[j]) << 8;
        }
        return (((n ^ this.numBands) << 8 ^ this.numBanks) << 8 ^ this.scanlineStride) << 8 ^ this.pixelStride;
    }
    
    static {
        ColorModel.loadLibraries();
        initIDs();
    }
}
